#
# The Zope Irc Bot
#
# (The Stupidest Zope Product Ever Written)
#
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
import time

from popen2 import popen3

from Persistence import Persistent
from Globals import ImageFile, HTMLFile, HTML, MessageDialog, package_home
from Acquisition import Implicit
from OFS.Folder import Folder
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse
from Products.ZSQLMethods.SQL import SQL

from Products.IrcBot.Plugins.Plugins import Plugin, PluginRegister
from Products.IrcBot.IrcChannel import IrcChannel

def manage_addFaqPlugin(self, id, title, connection,
						addLevel, modLevel,	delLevel,
						REQUEST={}):
	""" Add a Bot Faq Plugin """

	obj=FaqPlugin(id, connection, addLevel, modLevel, delLevel, title)
	self._setObject(id, obj)
	obj=getattr(self, id)
	obj.registerBotAction()

	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''



manage_addFaqPluginForm=HTMLFile('manage_addFaqPluginForm', globals())

class FaqPlugin(Plugin):
	""" A Faq restorer and retriever """

	manage_configurePluginForm=HTMLFile('manage_configureFaqPluginForm',
										globals())
	
	
	meta_type='FAQ Plugin'
	
	def __init__(self, id, connection,
				 addLevel, modLevel, delLevel, title=''):
		self.id = id
		self.title = title	
		self.connection = connection

		self.addLevel=addLevel
		self.modLevel=modLevel
		self.delLevel=delLevel
		self.addSQLQueries()

	def manage_configureFaqPlugin(self, connection,
								  addLevel, modLevel, delLevel):
		self.connection = connection

		self.addLevel=addLevel
		self.modLevel=modLevel
		self.delLevel=delLevel

	def addSQLQueries(self):
		self.sqlAddFAQ=SQL(
			'sqlAddFAQ',
			'Add a New FAQ',
			self.connection,
			'question answer committer',
			_sqlAddFAQ)

		self.sqlFindFAQ=SQL(
			'sqlFindFAQ',
			'Find a FAQ',
			self.connection,
			'key',
			_sqlFindFAQ)

		self.sqlShowFAQ=SQL(
			'sqlShowFAQ',
			'Show a FAQ by Id',
			self.connection,
			'faqid:int',
			_sqlShowFAQ)

		self.sqlDeleteFAQ=SQL(
			'sqlDeleteFAQ',
			'Delete a FAQ by Id',
			self.connection,
			'faqid:int',
			_sqlDeleteFAQ)

	def deregisterBotAction(self):
		self.ThreadMethods.addCommand(['UNBIND', 'PRIVMSG', self.add_faq], '^addfaq ')
		self.ThreadMethods.addCommand(['UNBIND', 'PRIVMSG', self.mod_faq], '^modfaq ')
		self.ThreadMethods.addCommand(['UNBIND', 'PRIVMSG', self.del_faq], '^delfaq ')				
		self.ThreadMethods.addCommand(['UNBIND', 'PRIVMSG', self.find_faq], '^find:')		
		self.ThreadMethods.addCommand(['UNBIND', 'PRIVMSG', self.list_faq], '^listfaq')
		self.ThreadMethods.addCommand(['UNBIND', 'PRIVMSG', self.show_faq], '^showfaq')		
		
	def registerBotAction(self):
		self.ThreadMethods.addCommand(['BIND', 'PRIVMSG', self.add_faq], '^addfaq ')
		self.ThreadMethods.addCommand(['BIND', 'PRIVMSG', self.mod_faq], '^modfaq ')
		self.ThreadMethods.addCommand(['BIND', 'PRIVMSG', self.del_faq], '^delfaq ')				
		self.ThreadMethods.addCommand(['BIND', 'PRIVMSG', self.find_faq], '^find:')		
		self.ThreadMethods.addCommand(['BIND', 'PRIVMSG', self.list_faq], '^listfaq')
		self.ThreadMethods.addCommand(['BIND', 'PRIVMSG', self.show_faq], '^showfaq')		
		

	# delfaq id:
	def del_faq(self, sender, recip, forum, addl, raw):
		# Only allow addition of faqs via /msg
		if forum is not sender:
			return

		if self.delLevel and (self.ThreadMethods.findUserLevel(raw) <
								  self.delLevel):
			sender.tell("AccessPrivs too low: Requires %d: Have %d"%(self.delLevel, self.findUserLevel(raw)))
			return
		
		command, args, text = raw
		text=string.strip(text[6:])
		faqid=0
		try:
			faqid=int(text)
		except:
			sender.tell("delfaq <id> where id is an integer")
			return

		self.sqlDeleteFAQ(faqid=faqid)
		sender.tell("FAQ: %d deleted"%(faqid))

	# modfaq id: question: answer
	def mod_faq(self, sender, recip, forum, addl, raw):
		pass
	# find: keyword
	def find_faq(self, sender, recip, forum, addl, raw):
		""" Search 'questions' for a relevant faq """
		command, args, text = raw
		# find: blah blah blah...
		text=string.strip(text[5:])

		faqs=self.sqlFindFAQ(key=text)
		if len(faqs) == 0:
			sender.tell("No FAQs Match '%s'"%(text))
			return

		if len(faqs)==1:
			for faq in faqs:
				question=sqlattr(faq, 'faqquestion')
				question=string.replace(question, text, '\002'+text+'\002')
				forum.tell('"%s" : %s'%(question, sqlattr(faq, 'faqanswer')))
				# forum.tell("Submitted By: %s"%(sqlattr(faq, 'faqcommitter')))
		else:
			numFaqs=len(faqs)
			sender.tell("There are %d faqs matching"%(numFaqs))
			sender.tell("Use showfaq <id> to select one (first 3 shown)")
			i=0
			for faq in faqs:
				i+=1
				sender.tell("%d: %s"%(sqlattr(faq, 'faqid'),
									 sqlattr(faq, 'faqquestion')))
				if i == 3:
					return
						  
	def list_faq(self, sender, recip, forum, addl, raw):
		pass
	def show_faq(self, sender, recip, forum, addl, raw):
		command, args, text = raw
		text = string.strip(text[7:])
		faqid=0
		try:
			faqid=int(text)
		except:
			sender.tell("Showfaq <id> where id is an integer")
			return

		faqs = self.sqlShowFAQ(faqid=faqid)
		if not len(faqs):
			sender.tell("No such faqid: %d"%(faqid))
		for faq in faqs:
			forum.tell("%s"%(sqlattr(faq, 'faqquestion')))				
			forum.tell("%s"%(sqlattr(faq, 'faqanswer')))
			# forum.tell("Submitted By: %s"%(sqlattr(faq, 'faqcommitter')))
		
	
	#
	# addfaq question: answer
	#
	def add_faq(self, sender, recip, forum, addl, raw):
		""" Add an FAQ """

		# Only allow addition of faqs via /msg
		if forum is not sender:
			return

		if self.addLevel and (self.ThreadMethods.findUserLevel(raw) <
								  self.addLevel):
			sender.tell("AccessPrivs too low: Requires %d: Have %d"%(self.addLevel, self.findUserLevel(raw)))
			return
		
		command, args, text = raw

		text=string.rstrip(text)
		faqadder=args[0] # Person who added the faq
		qna=text[7:]
		aindex=0
		try:
			aindex=string.index(qna,':')
		except:
			sender.tell("Error: addfaq question: answer")
			return

		if aindex < 1: # We should probably require a full word...
			sender.tell("Error: addfaq question: answer")
			return

		question=string.strip(qna[:aindex])
		answer=string.strip(qna[aindex+1:])

		self.sqlAddFAQ(question=question, answer=answer, committer=faqadder)
		sender.tell("Faq '%s' Added."%(question))
		return

from string import upper, lower
import Missing
mt=type(Missing.Value)

def typeconv(val):
    if type(val)==mt:
        return ''
    return val

def sqlattr(ob, attr):
    name=attr
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=upper(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=lower(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    raise NameError, name

_sqlAddFAQ='''
INSERT INTO faqs (faqquestion, faqanswer, faqcommitter)
VALUES (<dtml-sqlvar question type=string>,
        <dtml-sqlvar answer type=string>,
		<dtml-sqlvar committer type=string>)
'''

_sqlFindFAQ='''
SELECT * FROM faqs
WHERE faqquestion ~* '.*<dtml-var key sql_quote>.*'
'''

_sqlShowFAQ='''
SELECT * FROM faqs
WHERE faqid = <dtml-sqlvar faqid type=int>
'''

_sqlDeleteFAQ='''
DELETE FROM faqs
WHERE faqid = <dtml-sqlvar faqid type=int>
'''

FaqRegister=PluginRegister('Faq', 'Add and Find Questions and Answers',
						   FaqPlugin,
						   manage_addFaqPluginForm, manage_addFaqPlugin)

IrcChannel.Plugins['Faq']=FaqRegister
