#
# The Zope Irc Bot
#
# (The Stupidest Zope Product Ever Written)
#
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
import time

from popen2 import popen3

from Persistence import Persistent
from Globals import ImageFile, HTMLFile, HTML, MessageDialog, package_home
from Acquisition import Implicit
from OFS.Folder import Folder
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse

from Products.IrcBot.Plugins.Plugins import Plugin, PluginRegister
from Products.IrcBot.IrcChannel import IrcChannel

from FortuneData import FortuneData

def manage_addFortunePlugin(self, id, matchString, fortunePath,
							cacheTime, ignoreTime, commandLevel,
							REQUEST={}):

	""" Add the Fortune Plugin """

	obj=FortunePlugin(id=id, matchString=matchString, fortunePath=fortunePath,
					  cacheTime=cacheTime, ignoreTime=ignoreTime,
					  commandLevel=commandLevel)
	self._setObject(id, obj)
	obj=getattr(self, id)

	obj.enableBotAction()

	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''

manage_addFortunePluginForm=HTMLFile('manage_addFortunePluginForm',globals())

class FortunePlugin(Plugin):

	meta_type='Fortune Plugin'

	manage_configurePluginForm=HTMLFile('manage_configureFortunePluginForm',globals())

	
	def __init__(self, id, matchString, fortunePath,
				 cacheTime, ignoreTime, commandLevel=0):
		self.id = id
		self.title="Fortune Plugin"
		self.matchString=matchString
		self.fortunePath=fortunePath
		self.cacheTime=int(cacheTime)
		self.ignoreTime=int(ignoreTime)
		self.commandLevel=int(commandLevel)
		self.fortuneData=FortuneData()

	def manage_configureFortunePlugin(self, matchString, fortunePath,
									  cacheTime, ignoreTime, commandLevel,
									  REQUEST={}):
		""" Configure the Fortune Plugin """
		self.deregisterBotAction()
		self.matchString=matchString
		self.registerBotAction()
		
		self.fortunePath=fortunePath
		self.cacheTime=int(cacheTime)
		self.ignoreTime=int(ignoreTime)
		self.commandLevel=int(commandLevel)

		if REQUEST:
			return self.manage_configurePluginForm(self, REQUEST)
		return ''
		
	def registerBotAction(self):
		self.ThreadMethods.addCommand(['BIND', 'PRIVMSG', self.do_fortune], self.matchString)

	def deregisterBotAction(self):
		self.ThreadMethods.addCommand(['UNBIND', 'PRIVMSG', self.do_fortune], self.matchString)		
	def do_fortune(self, sender, recip, forum, addl, raw):

		if self.commandLevel and (self.ThreadMethods.findUserLevel(raw) < self.commandLevel):
			print "AccessPrivs too low: Requires %d: Have %d"%(self.commandLevel, self.findUserLevel(raw))
			return
		
		now = time.time()
		if (now - self.fortuneData.getLastTime()) < self.ignoreTime:
			sender.tell('%s was requested within %d seconds'%(self.id, self.ignoreTime))
			return
		
		if (now-self.fortuneData.getLastTime()) > self.cacheTime:
			self.fortuneData.clearFortuneList()
			stdout, stdin, stderr=popen3(self.fortunePath)
			for i in stdout.readlines():
				self.fortuneData.addFortuneLine(i)
			self.fortuneData.setLastTime(now)
			self._p_changed=1
		for line in self.fortuneData.getFortune():
			forum.tell(line)
			self.ThreadMethods.addLogEntry('', string.rstrip(line))
			time.sleep(0.7) # Don't flood


	bot_action = do_fortune
	
FortuneRegister=PluginRegister('Fortune', 'Show a random quote', FortunePlugin,
							   manage_addFortunePluginForm, manage_addFortunePlugin)

IrcChannel.Plugins['Fortune']=FortuneRegister
